#include <stdbool.h>
#include <stddef.h>
#include <sys/mman.h>
#include <unistd.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <stdint.h>

#include "align.h"
#include "chunk.h"
#include "chunk_manager.h"

int add_new_block(chunk_t* chunk, size_t size, void *prev, bool is_used)
{
	static int	block_id = 0;
	void		*new_block;
	chunk_t		previous_chunk;

	// TODO check with getrlimit()

	new_block = mmap(NULL, size + ALIGN_MARGING + HEADER_SIZE, PROT_READ | PROT_WRITE, MAP_PRIVATE | MAP_ANONYMOUS, -1, 0);
#ifndef DEBUG
	if (new_block == NULL)
		write(2, "mmap failed\n", 12);
#endif

	chunk_init(chunk, size + ALIGN_MARGING - get_align_increment(new_block + HEADER_SIZE), block_id, prev, new_block, NULL, is_used);
	
	if (prev != NULL)
	{
		chunk_read(prev, &previous_chunk);
		previous_chunk.next = chunk->current;
		chunk_write(&previous_chunk);
	}

	block_id++;

	return 0;
}

void destroy_block(chunk_t *chunk)
{
    chunk_t chunk_prev, chunk_next;
    
    if (chunk->prev != NULL)
    {
        chunk_read(chunk->prev, &chunk_prev);
        if (chunk_prev.block_id == chunk->block_id && chunk_prev.is_used == false)
            chunk_merge(&chunk_prev, chunk);
        chunk_prev.next = chunk->next;
        chunk_write(&chunk_prev);
    }
    if (chunk->next != NULL)
    {
        chunk_read(chunk->next, &chunk_next);
        if (chunk_next.block_id == chunk->block_id && chunk_next.is_used == false)
            chunk_merge(chunk, &chunk_next);
        chunk_next.prev = chunk->prev;
        chunk_write(&chunk_next);
    }

	munmap(chunk->current, get_physical_size(chunk));
}