#include <stddef.h>
#include <stdbool.h>
#include <unistd.h>

#include "./chunk.h"
#include "./align.h"
#include "print.h"

void chunk_write(const chunk_t *chunk)
{
	void **block = chunk->current;

	block[CHUNK_NEXT_POS] = (void*) chunk->next;
	block[CHUNK_CURRENT_POS] = (void*) chunk->current;
	block[CHUNK_PREV_POS] = (void*) chunk->prev;
	block[CHUNK_SIZE_POS] = (void*) chunk->size;
	block[CHUNK_IS_USED_POS] = (void*) chunk->is_used;
	block[CHUNK_BLOCK_ID_POS] = (void*) chunk->block_id;
	block[CHUNK_DATA_START_POS] = chunk->data_start;
}

void chunk_read(void * const *block, chunk_t *chunk)
{
    chunk->next = block[CHUNK_NEXT_POS];
	chunk->current = block[CHUNK_CURRENT_POS];
	chunk->prev = block[CHUNK_PREV_POS];
	chunk->size = (size_t) block[CHUNK_SIZE_POS];
	chunk->is_used = block[CHUNK_IS_USED_POS];
	chunk->block_id = (long) block[CHUNK_BLOCK_ID_POS];
	chunk->data_start = block[CHUNK_DATA_START_POS];
}

void chunk_init(chunk_t *chunk, size_t size, long block_id, void *prev, void *current, void *next, bool is_used)
{
	void	*data_start = current + HEADER_SIZE;

	data_start = data_start + get_align_increment(data_start);

	chunk->is_used = is_used;
	chunk->data_start = data_start;
	chunk->prev = prev;
	chunk->current = current;
	chunk->next = next;
	chunk->size = size;
	chunk->block_id = block_id;

	chunk_write(chunk);
}

size_t	get_physical_size(const chunk_t *chunk)
{
	return chunk->data_start + chunk->size - chunk->current;
}

void chunk_print(const chunk_t *chunk)
{
	putnbr_base((size_t) chunk->data_start, HEXA);
	putstr(" - ");
	putnbr_base((size_t) chunk->data_start + chunk->size, HEXA);
	putstr(" : ");
	putnbr_base(chunk->size, DECIMAL);
	putstr("bytes\n");
}